isdefined(Base, :__precompile__) && __precompile__()
module LITS

####################################### Structs Exports ####################################

# Base Exports
export DynamicLine
export Simulation
export run_simulation!
export ThreePhaseFault
export ControlReferenceChange
export DynamicLine

# Export for routines
export make_dynamic_branch!
export small_signal_analysis
export get_state_series
export get_voltagemag_series
export print_init_states

####################################### Package Imports ####################################
import InfrastructureSystems
import DiffEqBase
import ForwardDiff
import SparseArrays: SparseMatrixCSC
#import LinearAlgebra: BLAS
import LinearAlgebra: eigen
import Base.to_index
import NLsolve
import Base.ImmutableDict
import PowerSystems
const PSY = PowerSystems
const IS = InfrastructureSystems

#Structs for General Devices and System
include("base/definitions.jl")
include("base/ports.jl")
include("base/perturbations.jl")
include("base/small_signal_results.jl")
include("base/simulation.jl")

#Common Models
include("models/branch.jl")
include("models/device.jl")
include("models/kirchoff_laws.jl")
include("models/dynline_model.jl")
include("models/ref_transformations.jl")

#Generator Component Models
include("models/generator_models/machine_models.jl")
include("models/generator_models/pss_models.jl")
include("models/generator_models/avr_models.jl")
include("models/generator_models/tg_models.jl")
include("models/generator_models/shaft_models.jl")

#Inverter Component Models
include("models/inverter_models/DCside_models.jl")
include("models/inverter_models/filter_models.jl")
include("models/inverter_models/frequency_estimator_models.jl")
include("models/inverter_models/outer_control_models.jl")
include("models/inverter_models/voltage_source_control_models.jl")
include("models/inverter_models/converter_models.jl")

#Injection Models
include("models/load_models.jl")
include("models/source_models.jl")

#System Model
include("models/system.jl")

#Utils
include("utils/plot_utils.jl")
include("utils/immutable_dicts.jl")
include("utils/print.jl")
include("utils/kwargs_check.jl")

end # module
